//
// ExifTool 10.36 より。
/*
 改変

  LensType
    Zuiko Digital -> ZD
    M.Zuiko Digital -> MZD

  _LensCode

*/
// 
//
(function(){
 'use strict';
 const Ld = sLoader;
 Ld.module('exif-olympus',function(){

  const Olympus = {};
  const ExifReader = Ld.ref('exifReader');
  const U = Ld.use('utility');

  function x02(u){ return U.hex(u,2,'0'); }
  function trim(s) { return s.trim(); }

  const tagTable = {

   // Olympus Maker Note IFD
   Olympus: {
    0x0202: { name:'Macro' },           // u16 0:off, 1:on, 2:super macro
    0x0204: { name:'DigitalZoom' },     // ur64
    0x0207: { name:'_CameraCode1', conv:trim },
    0x0209: { name:'CameraId' },
    0x1000: { name:'ShutterSpeedValue', 
              conv: val=>val[1]>0? Math.pow(2,-val[0]/val[1]): 0
            },
    0x2010: { type:'IFD', group:'OlympusE' },
    0x2020: { type:'IFD', group:'OlympusCS' },
    0x2040: { type:'IFD', group:'OlympusIP' },
    0x2050: { type:'IFD', group:'OlympusFI' },
   },

   // Olympus Equipment
   OlympusE: {
    0x0100: { name:'_CameraCode2', conv:trim },
    0x0201: { name:'_LensCode',
              conv: d=>`${x02(d[0])}${x02(d[2])}${x02(d[3])}`
            },
    0x0203: { name:'LensModel' },
    0x0301: { name:'_ExtenderCode',
              conv: function(d){
               let code = `${x02(d[0])}${x02(d[2])}`;
               return code === '0000'? undefined: code;
              }
            },
    0x0403: { name:'ConversionLens' },
   },

   // Olympus Camera Setting
   OlympusCS: {
    0x0300: { name:'MacroMode' },       // u16 0:off, 1:on, 2:super macro
   },

   // Olympus Focus Info
   OlympusFI: {
    0x0000: { name:'FocusInfoVersion',
              conv: val=>ExifReader.dataViewToString(val)
            }, // undefined
    0x0300: { name:'ZoomStepCount'  },   // u16
    0x0301: { name:'FocusStepCount' },   // u16
    0x0303: { name:'FocusStepInfinity' },   // u16
    0x0304: { name:'FocusStepNear' },   // u16
   },

   OlympusIP: {
    // sensor sizeと思っていい?
    0x0614: { name: 'CropWidth' },
    0x0615: { name: 'CropHeight' },
   },

  };

  const olympusLensTypes = {
  
    '000000' : 'None',
    // Olympus lenses (also Kenko Tokina)
    '000100' : 'Olympus ZD ED 50mm F2.0 Macro',
    '000101' : 'Olympus ZD 40-150mm F3.5-4.5', //8
    '000110' : 'Olympus MZD ED 14-42mm F3.5-5.6', //PH (E-P1 pre-production)
    '000200' : 'Olympus ZD ED 150mm F2.0',
    '000210' : 'Olympus MZD 17mm F2.8 Pancake', //PH (E-P1 pre-production)
    '000300' : 'Olympus ZD ED 300mm F2.8',
    '000310' : 'Olympus MZD ED 14-150mm F4.0-5.6 [II]', //11 (The second version of this lens seems to have the same lens ID number as the first version #NJ)
    '000410' : 'Olympus MZD ED 9-18mm F4.0-5.6', //11
    '000500' : 'Olympus ZD 14-54mm F2.8-3.5',
    '000501' : 'Olympus ZD Pro ED 90-250mm F2.8', //9
    '000510' : 'Olympus MZD ED 14-42mm F3.5-5.6 L', //11 (E-PL1)
    '000600' : 'Olympus ZD ED 50-200mm F2.8-3.5',
    '000601' : 'Olympus ZD ED 8mm F3.5 Fisheye', //9
    '000610' : 'Olympus MZD ED 40-150mm F4.0-5.6', //PH
    '000700' : 'Olympus ZD 11-22mm F2.8-3.5',
    '000701' : 'Olympus ZD 18-180mm F3.5-6.3', //6
    '000710' : 'Olympus MZD ED 12mm F2.0', //PH
    '000801' : 'Olympus ZD 70-300mm F4.0-5.6', //7 (seen as release 1 - PH)
    '000810' : 'Olympus MZD ED 75-300mm F4.8-6.7', //PH
    '000910' : 'Olympus MZD 14-42mm F3.5-5.6 II', //PH (E-PL2)
    '001001' : 'Kenko Tokina Reflex 300mm F6.3 MF Macro', //NJ
    '001010' : 'Olympus MZD ED 12-50mm F3.5-6.3 EZ', //PH
    '001110' : 'Olympus MZD 45mm F1.8', //17
    '001210' : 'Olympus MZD ED 60mm F2.8 Macro', //NJ
    '001310' : 'Olympus MZD 14-42mm F3.5-5.6 II R', //PH/NJ
    '001410' : 'Olympus MZD ED 40-150mm F4.0-5.6 R', //19
  // '001410.1' : 'Olympus MZD ED 14-150mm F4.0-5.6 II', #11 (questionable & unconfirmed -- all samples I can find are '0 3 10' - PH)
    '001500' : 'Olympus ZD ED 7-14mm F4.0',
    '001510' : 'Olympus MZD ED 75mm F1.8', //PH
    '001610' : 'Olympus MZD 17mm F1.8', //NJ
    '001700' : 'Olympus ZD Pro ED 35-100mm F2.0', //7
    '001800' : 'Olympus ZD 14-45mm F3.5-5.6',
    '001810' : 'Olympus MZD ED 75-300mm F4.8-6.7 II', //NJ
    '001910' : 'Olympus MZD ED 12-40mm F2.8 Pro', //PH
    '002000' : 'Olympus ZD 35mm F3.5 Macro', //9
    '002010' : 'Olympus MZD ED 40-150mm F2.8 Pro', //NJ
    '002110' : 'Olympus MZD ED 14-42mm F3.5-5.6 EZ', //NJ
    '002200' : 'Olympus ZD 17.5-45mm F3.5-5.6', //9
    '002210' : 'Olympus MZD 25mm F1.8', //NJ
    '002300' : 'Olympus ZD ED 14-42mm F3.5-5.6', //PH
    '002310' : 'Olympus MZD ED 7-14mm F2.8 Pro', //NJ
    '002400' : 'Olympus ZD ED 40-150mm F4.0-5.6', //PH
    '002410' : 'Olympus MZD ED 300mm F4.0 IS Pro', //NJ
    '002510' : 'Olympus MZD ED 8mm F1.8 Fisheye Pro', //NJ
    '002610' : 'Olympus MZD ED 12-100mm F4.0 IS Pro', //IB/NJ
    '002710' : 'Olympus MZD ED 30mm F3.5 Macro', //IB/NJ
    '002810' : 'Olympus MZD ED 25mm F1.2 Pro', //IB/NJ
    '003000' : 'Olympus ZD ED 50-200mm F2.8-3.5 SWD', //7
    '003100' : 'Olympus ZD ED 12-60mm F2.8-4.0 SWD', //7
    '003200' : 'Olympus ZD ED 14-35mm F2.0 SWD', //PH
    '003300' : 'Olympus ZD 25mm F2.8', //PH
    '003400' : 'Olympus ZD ED 9-18mm F4.0-5.6', //7
    '003500' : 'Olympus ZD 14-54mm F2.8-3.5 II', //PH
    // Sigma lenses
    '010100' : 'Sigma 18-50mm F3.5-5.6 DC', //8
    '010110' : 'Sigma 30mm F2.8 EX DN', //NJ
    '010200' : 'Sigma 55-200mm F4.0-5.6 DC',
    '010210' : 'Sigma 19mm F2.8 EX DN', //NJ
    '010300' : 'Sigma 18-125mm F3.5-5.6 DC',
    '010310' : 'Sigma 30mm F2.8 DN | A', //NJ
    '010400' : 'Sigma 18-125mm F3.5-5.6 DC', //7
    '010410' : 'Sigma 19mm F2.8 DN | A', //NJ
    '010500' : 'Sigma 30mm F1.4 EX DC HSM', //10
    '010510' : 'Sigma 60mm F2.8 DN | A', //NJ
    '010600' : 'Sigma APO 50-500mm F4.0-6.3 EX DG HSM', //6
    '010610' : 'Sigma 30mm F1.4 DC DN | C', //NJ
    '010700' : 'Sigma Macro 105mm F2.8 EX DG', //PH
    '010800' : 'Sigma APO Macro 150mm F2.8 EX DG HSM', //PH
    '010900' : 'Sigma 18-50mm F2.8 EX DC Macro', //NJ
    '011000' : 'Sigma 24mm F1.8 EX DG Aspherical Macro', //PH
    '011100' : 'Sigma APO 135-400mm F4.5-5.6 DG', //11
    '011200' : 'Sigma APO 300-800mm F5.6 EX DG HSM', //11
    '011300' : 'Sigma 30mm F1.4 EX DC HSM', //11
    '011400' : 'Sigma APO 50-500mm F4.0-6.3 EX DG HSM', //11
    '011500' : 'Sigma 10-20mm F4.0-5.6 EX DC HSM', //11
    '011600' : 'Sigma APO 70-200mm F2.8 II EX DG Macro HSM', //11
    '011700' : 'Sigma 50mm F1.4 EX DG HSM', //11
    // Panasonic/Leica lenses
    '020100' : 'Leica D Vario Elmarit 14-50mm F2.8-3.5 Asph.', //11
    '020110' : 'Lumix G Vario 14-45mm F3.5-5.6 Asph. Mega OIS', //16
    '020200' : 'Leica D Summilux 25mm F1.4 Asph.', //11
    '020210' : 'Lumix G Vario 45-200mm F4.0-5.6 Mega OIS', //16
    '020300' : 'Leica D Vario Elmar 14-50mm F3.8-5.6 Asph. Mega OIS', //11
    '020301' : 'Leica D Vario Elmar 14-50mm F3.8-5.6 Asph.', //14 (L10 kit)
    '020310' : 'Lumix G Vario HD 14-140mm F4.0-5.8 Asph. Mega OIS', //16
    '020400' : 'Leica D Vario Elmar 14-150mm F3.5-5.6', //13
    '020410' : 'Lumix G Vario 7-14mm F4.0 Asph.', //PH (E-P1 pre-production)
    '020510' : 'Lumix G 20mm F1.7 Asph.', //16
    '020610' : 'Leica DG Macro-Elmarit 45mm F2.8 Asph. Mega OIS', //PH
    '020710' : 'Lumix G Vario 14-42mm F3.5-5.6 Asph. Mega OIS', //NJ
    '020810' : 'Lumix G Fisheye 8mm F3.5', //PH
    '020910' : 'Lumix G Vario 100-300mm F4.0-5.6 Mega OIS', //11
    '021010' : 'Lumix G 14mm F2.5 Asph.', //17
    '021110' : 'Lumix G 12.5mm F12 3D', //NJ (H-FT012)
    '021210' : 'Leica DG Summilux 25mm F1.4 Asph.', //NJ
    '021310' : 'Lumix G X Vario PZ 45-175mm F4.0-5.6 Asph. Power OIS', //NJ
    '021410' : 'Lumix G X Vario PZ 14-42mm F3.5-5.6 Asph. Power OIS', //NJ
    '021510' : 'Lumix G X Vario 12-35mm F2.8 Asph. Power OIS', //PH
    '021610' : 'Lumix G Vario 45-150mm F4.0-5.6 Asph. Mega OIS', //NJ
    '021710' : 'Lumix G X Vario 35-100mm F2.8 Power OIS', //PH
    '021810' : 'Lumix G Vario 14-42mm F3.5-5.6 II Asph. Mega OIS', //NJ
    '021910' : 'Lumix G Vario 14-140mm F3.5-5.6 Asph. Power OIS', //NJ
    '022010' : 'Lumix G Vario 12-32mm F3.5-5.6 Asph. Mega OIS', //NJ
    '022110' : 'Leica DG Nocticron 42.5mm F1.2 Asph. Power OIS', //NJ
    '022210' : 'Leica DG Summilux 15mm F1.7 Asph.', //NJ
    '022310' : 'Lumix G Vario 35-100mm F4.0-5.6 Asph. Mega OIS', //NJ
    '022410' : 'Lumix G Macro 30mm F2.8 Asph. Mega OIS', //NJ
    '022510' : 'Lumix G 42.5mm F1.7 Asph. Power OIS', //NJ
    '022610' : 'Lumix G 25mm F1.7 Asph.', //NJ
    '022710' : 'Leica DG Vario-Elmar 100-400mm F4.0-6.3 Asph. Power OIS', //NJ
    '022810' : 'Lumix G Vario 12-60mm F3.5-5.6 Asph. Power OIS', //NJ
    '030100' : 'Leica D Vario Elmarit 14-50mm F2.8-3.5 Asph.', //11
    '030200' : 'Leica D Summilux 25mm F1.4 Asph.', //11
    // Tamron lenses
    '050110' : 'Tamron 14-150mm F3.5-5.8 Di III', //NJ (model C001)
  };

  const olympusCameraTypes = {
    D4028 : 'X-2,C-50Z',
    D4029 : 'E-20,E-20N,E-20P',
    D4034 : 'C720UZ',
    D4040 : 'E-1',
    D4041 : 'E-300',
    D4083 : 'C2Z,D520Z,C220Z',
    D4106 : 'u20D,S400D,u400D',
    D4120 : 'X-1',
    D4122 : 'u10D,S300D,u300D',
    D4125 : 'AZ-1',
    D4141 : 'C150,D390',
    D4193 : 'C-5000Z',
    D4194 : 'X-3,C-60Z',
    D4199 : 'u30D,S410D,u410D',
    D4205 : 'X450,D535Z,C370Z',
    D4210 : 'C160,D395',
    D4211 : 'C725UZ',
    D4213 : 'FerrariMODEL2003',
    D4216 : 'u15D',
    D4217 : 'u25D',
    D4220 : 'u-miniD,Stylus V',
    D4221 : 'u40D,S500,uD500',
    D4231 : 'FerrariMODEL2004',
    D4240 : 'X500,D590Z,C470Z',
    D4244 : 'uD800,S800',
    D4256 : 'u720SW,S720SW',
    D4261 : 'X600,D630,FE5500',
    D4262 : 'uD600,S600',
    D4301 : 'u810/S810', // (yes, "/".  Olympus is not consistent in the notation)
    D4302 : 'u710,S710',
    D4303 : 'u700,S700',
    D4304 : 'FE100,X710',
    D4305 : 'FE110,X705',
    D4310 : 'FE-130,X-720',
    D4311 : 'FE-140,X-725',
    D4312 : 'FE150,X730',
    D4313 : 'FE160,X735',
    D4314 : 'u740,S740',
    D4315 : 'u750,S750',
    D4316 : 'u730/S730',
    D4317 : 'FE115,X715',
    D4321 : 'SP550UZ',
    D4322 : 'SP510UZ',
    D4324 : 'FE170,X760',
    D4326 : 'FE200',
    D4327 : 'FE190/X750', // (also SX876)
    D4328 : 'u760,S760',
    D4330 : 'FE180/X745', // (also SX875)
    D4331 : 'u1000/S1000',
    D4332 : 'u770SW,S770SW',
    D4333 : 'FE240/X795',
    D4334 : 'FE210,X775',
    D4336 : 'FE230/X790',
    D4337 : 'FE220,X785',
    D4338 : 'u725SW,S725SW',
    D4339 : 'FE250/X800',
    D4341 : 'u780,S780',
    D4343 : 'u790SW,S790SW',
    D4344 : 'u1020,S1020',
    D4346 : 'FE15,X10',
    D4348 : 'FE280,X820,C520',
    D4349 : 'FE300,X830',
    D4350 : 'u820,S820',
    D4351 : 'u1200,S1200',
    D4352 : 'FE270,X815,C510',
    D4353 : 'u795SW,S795SW',
    D4354 : 'u1030SW,S1030SW',
    D4355 : 'SP560UZ',
    D4356 : 'u1010,S1010',
    D4357 : 'u830,S830',
    D4359 : 'u840,S840',
    D4360 : 'FE350WIDE,X865',
    D4361 : 'u850SW,S850SW',
    D4362 : 'FE340,X855,C560',
    D4363 : 'FE320,X835,C540',
    D4364 : 'SP570UZ',
    D4366 : 'FE330,X845,C550',
    D4368 : 'FE310,X840,C530',
    D4370 : 'u1050SW,S1050SW',
    D4371 : 'u1060,S1060',
    D4372 : 'FE370,X880,C575',
    D4374 : 'SP565UZ',
    D4377 : 'u1040,S1040',
    D4378 : 'FE360,X875,C570',
    D4379 : 'FE20,X15,C25',
    D4380 : 'uT6000,ST6000',
    D4381 : 'uT8000,ST8000',
    D4382 : 'u9000,S9000',
    D4384 : 'SP590UZ',
    D4385 : 'FE3010,X895',
    D4386 : 'FE3000,X890',
    D4387 : 'FE35,X30',
    D4388 : 'u550WP,S550WP',
    D4390 : 'FE5000,X905',
    D4391 : 'u5000',
    D4392 : 'u7000,S7000',
    D4396 : 'FE5010,X915',
    D4397 : 'FE25,X20',
    D4398 : 'FE45,X40',
    D4401 : 'XZ-1',
    D4402 : 'uT6010,ST6010',
    D4406 : 'u7010,S7010 / u7020,S7020',
    D4407 : 'FE4010,X930',
    D4408 : 'X560WP',
    D4409 : 'FE26,X21',
    D4410 : 'FE4000,X920,X925',
    D4411 : 'FE46,X41,X42',
    D4412 : 'FE5020,X935',
    D4413 : 'uTough-3000',
    D4414 : 'StylusTough-6020',
    D4415 : 'StylusTough-8010',
    D4417 : 'u5010,S5010',
    D4418 : 'u7040,S7040',
    D4419 : 'u9010,S9010',
    D4423 : 'FE4040',
    D4424 : 'FE47,X43',
    D4426 : 'FE4030,X950',
    D4428 : 'FE5030,X965,X960',
    D4430 : 'u7030,S7030',
    D4432 : 'SP600UZ',
    D4434 : 'SP800UZ',
    D4439 : 'FE4020,X940',
    D4442 : 'FE5035',
    D4448 : 'FE4050,X970',
    D4450 : 'FE5050,X985',
    D4454 : 'u-7050',
    D4464 : 'T10,X27',
    D4470 : 'FE5040,X980',
    D4472 : 'TG-310',
    D4474 : 'TG-610',
    D4476 : 'TG-810',
    D4478 : 'VG145,VG140,D715',
    D4479 : 'VG130,D710',
    D4480 : 'VG120,D705',
    D4482 : 'VR310,D720',
    D4484 : 'VR320,D725',
    D4486 : 'VR330,D730',
    D4488 : 'VG110,D700',
    D4490 : 'SP-610UZ',
    D4492 : 'SZ-10',
    D4494 : 'SZ-20',
    D4496 : 'SZ-30MR',
    D4498 : 'SP-810UZ',
    D4500 : 'SZ-11',
    D4504 : 'TG-615',
    D4508 : 'TG-620',
    D4510 : 'TG-820',
    D4512 : 'TG-1',
    D4516 : 'SH-21',
    D4519 : 'SZ-14',
    D4520 : 'SZ-31MR',
    D4521 : 'SH-25MR',
    D4523 : 'SP-720UZ',
    D4529 : 'VG170',
    D4531 : 'XZ-2',
    D4535 : 'SP-620UZ',
    D4536 : 'TG-320',
    D4537 : 'VR340,D750',
    D4538 : 'VG160,X990,D745',
    D4541 : 'SZ-12',
    D4545 : 'VH410',
    D4546 : 'XZ-10', //IB
    D4547 : 'TG-2',
    D4548 : 'TG-830',
    D4549 : 'TG-630',
    D4550 : 'SH-50',
    D4553 : 'SZ-16,DZ-105',
    D4562 : 'SP-820UZ',
    D4566 : 'SZ-15',
    D4572 : 'STYLUS1',
    D4574 : 'TG-3',
    D4575 : 'TG-850',
    D4579 : 'SP-100EE',
    D4580 : 'SH-60',
    D4581 : 'SH-1',
    D4582 : 'TG-835',
    D4585 : 'SH-2 / SH-3',
    D4586 : 'TG-4',
    D4587 : 'TG-860',
    D4591 : 'TG-870',
    D4593 : 'TG-5', //IB
    D4809 : 'C2500L',
    D4842 : 'E-10',
    D4856 : 'C-1',
    D4857 : 'C-1Z,D-150Z',
    DCHC : 'D500L',
    DCHT : 'D600L / D620L',
    K0055 : 'AIR-A01',
    S0003 : 'E-330',
    S0004 : 'E-500',
    S0009 : 'E-400',
    S0010 : 'E-510',
    S0011 : 'E-3',
    S0013 : 'E-410',
    S0016 : 'E-420',
    S0017 : 'E-30',
    S0018 : 'E-520',
    S0019 : 'E-P1',
    S0023 : 'E-620',
    S0026 : 'E-P2',
    S0027 : 'E-PL1',
    S0029 : 'E-450',
    S0030 : 'E-600',
    S0032 : 'E-P3',
    S0033 : 'E-5',
    S0034 : 'E-PL2',
    S0036 : 'E-M5',
    S0038 : 'E-PL3',
    S0039 : 'E-PM1',
    S0040 : 'E-PL1s',
    S0042 : 'E-PL5',
    S0043 : 'E-PM2',
    S0044 : 'E-P5',
    S0045 : 'E-PL6',
    S0046 : 'E-PL7', //IB
    S0047 : 'E-M1',
    S0051 : 'E-M10',
    S0052 : 'E-M5MarkII', //IB
    S0059 : 'E-M10MarkII',
    S0061 : 'PEN-F', //forum7005
    S0065 : 'E-PL8',
    S0067 : 'E-M1MarkII',
    SR45 : 'D220',
    SR55 : 'D320L',
    SR83 : 'D340L',
    SR85 : 'C830L,D340R',
    SR852 : 'C860L,D360L',
    SR872 : 'C900Z,D400Z',
    SR874 : 'C960Z,D460Z',
    SR951 : 'C2000Z',
    SR952 : 'C21',
    SR953 : 'C21T.commu',
    SR954 : 'C2020Z',
    SR955 : 'C990Z,D490Z',
    SR956 : 'C211Z',
    SR959 : 'C990ZS,D490Z',
    SR95A : 'C2100UZ',
    SR971 : 'C100,D370',
    SR973 : 'C2,D230',
    SX151 : 'E100RS',
    SX351 : 'C3000Z / C3030Z',
    SX354 : 'C3040Z',
    SX355 : 'C2040Z',
    SX357 : 'C700UZ',
    SX358 : 'C200Z,D510Z',
    SX374 : 'C3100Z,C3020Z',
    SX552 : 'C4040Z',
    SX553 : 'C40Z,D40Z',
    SX556 : 'C730UZ',
    SX558 : 'C5050Z',
    SX571 : 'C120,D380',
    SX574 : 'C300Z,D550Z',
    SX575 : 'C4100Z,C4000Z',
    SX751 : 'X200,D560Z,C350Z',
    SX752 : 'X300,D565Z,C450Z',
    SX753 : 'C750UZ',
    SX754 : 'C740UZ',
    SX755 : 'C755UZ',
    SX756 : 'C5060WZ',
    SX757 : 'C8080WZ',
    SX758 : 'X350,D575Z,C360Z',
    SX759 : 'X400,D580Z,C460Z',
    SX75A : 'AZ-2ZOOM',
    SX75B : 'D595Z,C500Z',
    SX75C : 'X550,D545Z,C480Z',
    SX75D : 'IR-300',
    SX75F : 'C55Z,C5500Z',
    SX75G : 'C170,D425',
    SX75J : 'C180,D435',
    SX771 : 'C760UZ',
    SX772 : 'C770UZ',
    SX773 : 'C745UZ',
    SX774 : 'X250,D560Z,C350Z',
    SX775 : 'X100,D540Z,C310Z',
    SX776 : 'C460ZdelSol',
    SX777 : 'C765UZ',
    SX77A : 'D555Z,C315Z',
    SX851 : 'C7070WZ',
    SX852 : 'C70Z,C7000Z',
    SX853 : 'SP500UZ',
    SX854 : 'SP310',
    SX855 : 'SP350',
    SX873 : 'SP320',
    SX875 : 'FE180/X745', // (also D4330)
    SX876 : 'FE190/X750', // (also D4327)
//   other brands
//    4MP9Q3 : 'Camera 4MP-9Q3'
//    4MP9T2 : 'BenQ DC C420 / Camera 4MP-9T2'
//    5MP9Q3 : 'Camera 5MP-9Q3',
//    5MP9X9 : 'Camera 5MP-9X9',
//   '5MP-9T': 'Camera 5MP-9T3',
//   '5MP-9Y': 'Camera 5MP-9Y2',
//   '6MP-9U': 'Camera 6MP-9U9',
//    7MP9Q3 : 'Camera 7MP-9Q3',
//   '8MP-9U': 'Camera 8MP-9U4',
//    CE5330 : 'Acer CE-5330',
//   'CP-853': 'Acer CP-8531',
//    CS5531 : 'Acer CS5531',
//    DC500  : 'SeaLife DC500',
//    DC7370 : 'Camera 7MP-9GA',
//    DC7371 : 'Camera 7MP-9GM',
//    DC7371 : 'Hitachi HDC-751E',
//    DC7375 : 'Hitachi HDC-763E / Rollei RCP-7330X / Ricoh Caplio RR770 / Vivitar ViviCam 7330',
//   'DC E63': 'BenQ DC E63+',
//   'DC P86': 'BenQ DC P860',
//    DS5340 : 'Maginon Performic S5 / Premier 5MP-9M7',
//    DS5341 : 'BenQ E53+ / Supra TCM X50 / Maginon X50 / Premier 5MP-9P8',
//    DS5346 : 'Premier 5MP-9Q2',
//    E500   : 'Konica Minolta DiMAGE E500',
//    MAGINO : 'Maginon X60',
//    Mz60   : 'HP Photosmart Mz60',
//    Q3DIGI : 'Camera 5MP-9Q3',
//    SLIMLI : 'Supra Slimline X6',
//    V8300s : 'Vivitar V8300s',
  };

  const olympusExtenders = {
   '0000' : undefined,
   '0004' : 'Olympus ZD EC-14',
   '0008' : 'Olympus EX-25',
   '0010' : 'Olympus ZD EC-20', //7
  };

  const converterMagnification = {
   '0000' : 1,
   '0004' : 1.414,
   '0008' : 0,          // 
   '0010' : 2.0,
  };


  Olympus.readMakerNote = function(exif,data){
   let sig = ExifReader.dataViewToString(data,8);
   let offset, group;
   if ( sig.match(/^OLYMPUS\0/) ) {
    // Olympus2
    offset = 12;
    group = 'Olympus';
   } else {
    return;
   }

   let littleEndian = data.getUint16(8) === 0x4949;
   ExifReader.processExif( exif, data, offset, tagTable, group, littleEndian );

// FocusInfoVersion === '0100' !!
   exif._MakerCode = 'olympus';
   exif._CameraCode = exif._CameraCode1 || exif._CameraCode2;
   exif.CameraType = olympusCameraTypes[exif._CameraCode];
   exif._CameraClass = exif._CameraCode.match(/^S\d/)
     ? 'FourThirds': esc(exif._CameraCode);

   let d, key;
   d = exif._LensCode;
   if ( d ) {
    exif.LensType = olympusLensTypes[d];
   }

   if ( exif._ExtenderCode ) {
    let extender = olympusExtenders[exif._ExtenderCode];
    if ( extender ) { exif.Extender = extender; }
// 0の処理
    // 0: extenstion tube or unknown
    let m = converterMagnification[exif._ExtenderCode];
    exif._ConverterMagnification = m === undefined? 0: m;
   }

   const modeTable = {
    'N' : 'Normal mode',
    'M1': 'Macro mode',
    'M2': 'Super Macro mode',
   };

   // 'N': normal mode
   // 'M1': macro mode
   // 'M2': super macro mode
   let macroMode = exif.MacroMode || exif.Macro;
   if ( macroMode ) {
    exif._ModeCode = 'M'+macroMode;
   }
   else {
    exif._ModeCode = 'N';
   }
   exif._Mode = modeTable[exif._ModeCode];
   if ( exif._Mode === undefined ) {
    exif._Mode = 'unknown '+exif._ModeCode;
   }
   return true;
  };


  // a-zA-Z0-9_
  function esc(s){
   s = encodeURIComponent(s);
   return s.replace(/([^a-zA-Z0-9_\%])/g,function(s,p1){
    return '%'+x02(p1.charCodeAt(0));
   });
  }


/*
  Olympus.getCameraType = function(cameraCode){
   let type;
   if ( cameraCode.match(/^S\d/) ) {
    type = 'FourThirds';
     // contain Micro Four Thirds
   } else {
    type = esc(cameraCode);
   }
   return type;
  };
*/

  return Olympus;

 });
})();
