/*
 *  wm-image.js
 */
(function(){
 'use strict';
 const Ld = sLoader;
 Ld.module('wm-image',function(){
  const Wm = Ld.use('waterMark');
  const WmI = Wm.derive();

  const M  = Ld.ref('main');
  const U  = Ld.use('utility');
  const Mn = Ld.use('menu');
  const Cp = Ld.use('cropParam');
  const Mb = Ld.use('messageBox');

  WmI._className = 'wmImage';
  WmI.sig = 'WMImage';

  WmI.defineMenu = function(){
   WmI._super.defineMenu.call(this);

   let obj = this;

   this.addMenu({
    image         : Mn.button({priority:30,
                     text:'Import<br>Image',
                     onclick: ev=>{ obj.importImage(); }
                    }),

    // 普通のラジオボタンみたいなもの
    makeAlpha    : Mn.CSelect.create({priority:20,
      title: 'Make Alpha',
      default: false,
      list: [
       [ false, 'none' ],
// 幅超えるとエラーになるのなんとか
       [ true , 'Britness to Alpha Channel' ]
      ],
     }),
   });

  };


  WmI.importImage = function(){
   (async()=>{
    let file = (await U.openFileP(
     {accept:'image/*', name:'files[]'}))[0];
    if ( file ) {
     // 1MB  4/3->Base64 
     const FILE_SIZE_LIMIT = 1024*1024*4/3*1.1+100;
     if ( file.size > FILE_SIZE_LIMIT ) {
      Mb.messageBox({
       text:
        `<h2>ERROR: File Size Exceeded</h2>
        <p>The maximum size of Watermark image is 1MB.</p>`
      });
     } else {
      try{
       M.startLoadingEffect();
       let dataURL = await U.readAsDataURLP(file);
       let image = await U.loadImageP(file);
       this.values.dataURL = dataURL;
       this.values.imageWidth = image.width;
       this.values.imageHeight = image.height;
       M.stopLoadingEffect();
       this.setRasterImage(image);
      }
      catch(e){
       M.error(e);
       M.stopLoadingEffect();
      }
     }
    }
   })();
  };


  WmI.setImage = function(){
   if ( !this.values.dataURL ) { return; }
   let obj = this;
   U.dataURLToImageP(this.values.dataURL)
   .then((image)=>{
    obj.setRasterImage(image);
   })
   .catch((e)=>{
//
    console.log(e);
    obj.imageReady = false;
    obj.dirty = 1;
    M.disp();
   });
  };


  WmI.setRasterImage = function(image){
   image.width = this.values.imageWidth;
   image.height = this.values.imageHeight;
   this.image = image;
   this.imageReady = true;
   let el = this.imageContainer;
   if ( !el ) {
    el = this.imageContainer = document.createElement('div');
    U.setStyle(el.style,{
     visibility:'hidden',
    });
// 
    document.body.appendChild(el);
   }
   el.innerHTML = `
    <svg viewBox="0 0 ${image.width} ${image.height}"
      xmlns="http://www.w3.org/2000/svg"'
      xmlns:xlink="http://www.w3.org/1999/xlink" >
     <defs>
     <image id="wmimage" xlink:href="${this.values.dataURL}"
      x="0" y="0" width="${image.width}" height="${image.height}" />
     </defs>
    </svg>`;
   this.dirty = 1;
   M.disp();
  };

  WmI.getSourceBoundingBox = function(){
   return this.getGeometry();
  };

  WmI.getHighlightBox = function(){
   return this.getGeometry();
  };

  WmI.getGeometry = function(){
   let V = this.values;
   if ( !M.imageReady || !V.show || !this.imageReady ) { return undefined; }
   let sizeBase = Cp.width/Cp.diagonal;
   let size = sizeBase*V.transform.zoom/100;
   let iw = this.values.imageWidth;
   let ih = this.values.imageHeight;

// 高さ
   let bw = V.transform.zoom*sizeBase/100;
   let bh = bw*ih/iw;
   let x = V.transform.x /* *Cp.width/Cp.diagonal */ - bw/2;
   let y = V.transform.y /* *Cp.height/Cp.diagonal */ - bh/2;
if ( isNaN(y) ) throw "#5521";

   let cw = Cp.width/Cp.diagonal/2;
   let ch = Cp.height/Cp.diagonal/2;
   if ( x < -cw ) { x = -cw; }
   if ( x + bw > cw ) { x = cw - bw; }
   if ( y < -ch ) { y = -ch; }
   if ( y + bh > ch ) { y = ch - bh; }
   return {
    x           : x,
    y           : y,
    width       : bw,
    height      : bh,
    centerX     : x+bw/2,
    centerY     : y+bh/2
   };
  };


// final/editMode
  WmI.makeSvgTextContent = function(final,scale){
   let geom = this.getGeometry();
   let V = this.values;
   let s = geom.width*scale/this.values.imageWidth;
   let svg =
    `<g transform="translate(${scale*geom.x} ${scale*geom.y}) scale(${s})"
      width="${this.values.imageWidth}" height="${this.values.imageHeight}" >`;
   if ( final ) {
    svg += `<image xlink:href="${this.values.dataURL}" x="0" y="0"
     width="${this.values.imageWidth}" height="${this.values.imageHeight}" />`;
   } else {
    svg += `<use xlink:href="#wmimage" x="0" y="0" />`;
   }
   svg += '</g>';
   return svg;
  };


  WmI.restoreSessionStatus = function(data){
   WmI._super.restoreSessionStatus.call(this,data);
   if ( data ) {
    this.values.dataURL = data.dataURL;
    this.values.imageWidth = data.imageWidth;
    this.values.imageHeight = data.imageHeight;
    this.setImage();
   }
  };


  WmI.loadSetting = function(data){
   WmI._super.loadSetting.call(this,data);
   if ( data ) {
    this.values.dataURL = data.dataURL;
    this.values.imageWidth = data.imageWidth;
    this.values.imageHeight = data.imageHeight;
    this.setImage();
   }
  };


  return WmI;
 });
})();
